@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 BogDan Vatra <bog_dan_ro@yahoo.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 BogDan Vatra <bog_dan_ro@yahoo.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #include "androiddebugsupport.h"
 
@@ -44,7 +22,6 @@
 
 #include <utils/hostosinfo.h>
 
-#include <QDirIterator>
 #include <QHostAddress>
 #include <QJsonDocument>
 #include <QLoggingCategory>
@@ -123,7 +100,7 @@ void AndroidDebugSupport::start()
     setUseContinueInsteadOfRun(true);
     setAttachPid(m_runner->pid());
 
-    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit);
+    QtSupport::QtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit);
     if (!Utils::HostOsInfo::isWindowsHost()
         && (qtVersion
             && AndroidConfigurations::currentConfig().ndkVersion(qtVersion)
@@ -141,27 +118,38 @@ void AndroidDebugSupport::start()
             solibSearchPath.append(qtVersion->qtSoPaths());
         solibSearchPath.append(uniquePaths(extraLibs));
 
+        FilePath buildDir = AndroidManager::buildDirectory(target);
         const RunConfiguration *activeRunConfig = target->activeRunConfiguration();
-        FilePath buildDir;
         if (activeRunConfig)
-            buildDir = activeRunConfig->buildTargetInfo().workingDirectory;
+            solibSearchPath.append(activeRunConfig->buildTargetInfo().workingDirectory.toString());
         solibSearchPath.append(buildDir.toString());
+        const auto androidLibsPath = AndroidManager::androidBuildDirectory(target)
+                                         .pathAppended("libs")
+                                         .pathAppended(AndroidManager::apkDevicePreferredAbi(target))
+                                         .toString();
+        solibSearchPath.append(androidLibsPath);
         solibSearchPath.removeDuplicates();
         setSolibSearchPath(solibSearchPath);
-        qCDebug(androidDebugSupportLog) << "SoLibSearchPath: "<<solibSearchPath;
+        qCDebug(androidDebugSupportLog).noquote() << "SoLibSearchPath: " << solibSearchPath;
         setSymbolFile(buildDir.pathAppended("app_process"));
         setSkipExecutableValidation(true);
         setUseExtendedRemote(true);
         QString devicePreferredAbi = AndroidManager::apkDevicePreferredAbi(target);
         setAbi(AndroidManager::androidAbi2Abi(devicePreferredAbi));
 
-        QUrl debugServer;
-        debugServer.setPort(m_runner->debugServerPort().number());
         if (cppEngineType() == LldbEngineType) {
-            debugServer.setScheme("adb");
-            debugServer.setHost(AndroidManager::deviceSerialNumber(target));
-            setRemoteChannel(debugServer.toString());
+            QString deviceSerialNumber = AndroidManager::deviceSerialNumber(target);
+            const int colonPos = deviceSerialNumber.indexOf(QLatin1Char(':'));
+            if (colonPos > 0) {
+                // When wireless debugging is used then the device serial number will include a port number
+                // The port number must be removed to form a valid hostname
+                deviceSerialNumber.truncate(colonPos);
+            }
+            setRemoteChannel("adb://" + deviceSerialNumber,
+                             m_runner->debugServerPort().number());
         } else {
+            QUrl debugServer;
+            debugServer.setPort(m_runner->debugServerPort().number());
             debugServer.setHost(QHostAddress(QHostAddress::LocalHost).toString());
             setRemoteChannel(debugServer);
         }
@@ -170,16 +158,17 @@ void AndroidDebugSupport::start()
         const int minimumNdk = qt ? qt->minimumNDK() : 0;
 
         int sdkVersion = qMax(AndroidManager::minimumSDK(kit), minimumNdk);
-        // TODO find a way to use the new sysroot layout
-        // instead ~/android/ndk-bundle/platforms/android-29/arch-arm64
-        // use ~/android/ndk-bundle/toolchains/llvm/prebuilt/linux-x86_64/sysroot
         if (qtVersion) {
-            Utils::FilePath sysRoot = AndroidConfigurations::currentConfig().ndkLocation(qtVersion)
+            const FilePath ndkLocation =
+                    AndroidConfigurations::currentConfig().ndkLocation(qtVersion);
+            Utils::FilePath sysRoot = ndkLocation
                     / "platforms"
                     / QString("android-%1").arg(sdkVersion)
-                    / devicePreferredAbi;
+                    / devicePreferredAbi; // Legacy Ndk structure
+            if (!sysRoot.exists())
+                sysRoot = AndroidConfig::toolchainPathFromNdk(ndkLocation) / "sysroot";
             setSysRoot(sysRoot);
-            qCDebug(androidDebugSupportLog) << "Sysroot: " << sysRoot;
+            qCDebug(androidDebugSupportLog).noquote() << "Sysroot: " << sysRoot.toUserOutput();
         }
     }
     if (isQmlDebugging()) {
