@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #include "formeditorwidget.h"
 #include "designeractionmanager.h"
@@ -32,14 +10,15 @@
 #include "qmldesignericons.h"
 #include "qmldesignerplugin.h"
 #include "viewmanager.h"
-#include <model.h>
-#include <theme.h>
 
+#include <auxiliarydataproperties.h>
 #include <backgroundaction.h>
 #include <formeditorgraphicsview.h>
 #include <formeditorscene.h>
 #include <formeditorview.h>
 #include <lineeditaction.h>
+#include <model.h>
+#include <theme.h>
 #include <toolbox.h>
 #include <zoomaction.h>
 
@@ -53,6 +32,7 @@
 
 #include <QActionGroup>
 #include <QFileDialog>
+#include <QMimeData>
 #include <QPainter>
 #include <QPicture>
 #include <QVBoxLayout>
@@ -60,9 +40,16 @@
 
 namespace QmlDesigner {
 
+namespace {
+constexpr AuxiliaryDataKeyView formeditorZoomProperty{AuxiliaryDataType::NodeInstancePropertyOverwrite,
+                                                      "formeditorZoom"};
+}
+
 FormEditorWidget::FormEditorWidget(FormEditorView *view)
     : m_formEditorView(view)
 {
+    setAcceptDrops(true);
+
     Core::Context context(Constants::C_QMLFORMEDITOR);
     m_context = new Core::IContext(this);
     m_context->setContext(context);
@@ -87,13 +74,13 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
     m_noSnappingAction->setIcon(Icons::NO_SNAPPING.icon());
     registerActionAsCommand(m_noSnappingAction, Constants::FORMEDITOR_NO_SNAPPING, QKeySequence(Qt::Key_T));
 
-    m_snappingAndAnchoringAction = layoutActionGroup->addAction(tr("Snap to parent or sibling items and generate anchors."));
+    m_snappingAndAnchoringAction = layoutActionGroup->addAction(tr("Snap to parent or sibling components and generate anchors."));
     m_snappingAndAnchoringAction->setCheckable(true);
     m_snappingAndAnchoringAction->setChecked(true);
     m_snappingAndAnchoringAction->setIcon(Icons::NO_SNAPPING_AND_ANCHORING.icon());
     registerActionAsCommand(m_snappingAndAnchoringAction, Constants::FORMEDITOR_NO_SNAPPING_AND_ANCHORING, QKeySequence(Qt::Key_W));
 
-    m_snappingAction = layoutActionGroup->addAction(tr("Snap to parent or sibling items but do not generate anchors."));
+    m_snappingAction = layoutActionGroup->addAction(tr("Snap to parent or sibling components but do not generate anchors."));
     m_snappingAction->setCheckable(true);
     m_snappingAction->setChecked(true);
     m_snappingAction->setIcon(Icons::SNAPPING.icon());
@@ -108,7 +95,7 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
     upperActions.append(separatorAction);
 
     m_showBoundingRectAction = new QAction(Utils::Icons::BOUNDING_RECT.icon(),
-                                           tr("Show bounding rectangles and stripes for empty items."),
+                                           tr("Show bounding rectangles and stripes for empty components."),
                                            this);
     m_showBoundingRectAction->setCheckable(true);
     m_showBoundingRectAction->setChecked(false);
@@ -123,20 +110,20 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
     upperActions.append(separatorAction);
 
     m_rootWidthAction = new LineEditAction(tr("Override Width"), this);
-    m_rootWidthAction->setToolTip(tr("Override width of root item."));
+    m_rootWidthAction->setToolTip(tr("Override width of root component."));
     connect(m_rootWidthAction.data(), &LineEditAction::textChanged,
             this, &FormEditorWidget::changeRootItemWidth);
     addAction(m_rootWidthAction.data());
     upperActions.append(m_rootWidthAction.data());
 
     m_rootHeightAction = new LineEditAction(tr("Override Height"), this);
-    m_rootHeightAction->setToolTip(tr("Override height of root item."));
+    m_rootHeightAction->setToolTip(tr("Override height of root component."));
     connect(m_rootHeightAction.data(), &LineEditAction::textChanged,
             this, &FormEditorWidget::changeRootItemHeight);
     addAction(m_rootHeightAction.data());
     upperActions.append(m_rootHeightAction.data());
 
-    m_toolBox = new ToolBox(this);
+    m_toolBox = new ToolBox(nullptr, this);
     fillLayout->addWidget(m_toolBox.data());
 
     m_toolBox->setLeftSideActions(upperActions);
@@ -149,18 +136,18 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
 
     // Zoom actions
     const QString fontName = "qtds_propertyIconFont.ttf";
-    const QColor textColorNormal(Theme::getColor(Theme::MenuItemTextColorNormal));
-    const QColor textColorDisabled(Theme::getColor(Theme::MenuBarItemTextColorDisabled));
+    const QColor iconColorNormal(Theme::getColor(Theme::IconsBaseColor));
+    const QColor iconColorDisabled(Theme::getColor(Theme::IconsDisabledColor));
     const QIcon zoomAllIcon = Utils::StyleHelper::getIconFromIconFont(
-        fontName, Theme::getIconUnicode(Theme::Icon::zoomAll), 28, 28, textColorNormal);
+        fontName, Theme::getIconUnicode(Theme::Icon::zoomAll), 28, 28, iconColorNormal);
 
     const QString zoomSelectionUnicode = Theme::getIconUnicode(Theme::Icon::zoomSelection);
     const auto zoomSelectionNormal = Utils::StyleHelper::IconFontHelper(zoomSelectionUnicode,
-                                                                        textColorNormal,
+                                                                        iconColorNormal,
                                                                         QSize(28, 28),
                                                                         QIcon::Normal);
     const auto zoomSelectionDisabeld = Utils::StyleHelper::IconFontHelper(zoomSelectionUnicode,
-                                                                          textColorDisabled,
+                                                                          iconColorDisabled,
                                                                           QSize(28, 28),
                                                                           QIcon::Disabled);
 
@@ -168,17 +155,16 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
                                                                             {zoomSelectionNormal,
                                                                              zoomSelectionDisabeld});
     const QIcon zoomInIcon = Utils::StyleHelper::getIconFromIconFont(
-        fontName, Theme::getIconUnicode(Theme::Icon::zoomIn), 28, 28, textColorNormal);
+        fontName, Theme::getIconUnicode(Theme::Icon::zoomIn), 28, 28, iconColorNormal);
     const QIcon zoomOutIcon = Utils::StyleHelper::getIconFromIconFont(
-        fontName, Theme::getIconUnicode(Theme::Icon::zoomOut), 28, 28, textColorNormal);
+        fontName, Theme::getIconUnicode(Theme::Icon::zoomOut), 28, 28, iconColorNormal);
 
     auto writeZoomLevel = [this]() {
         double level = m_graphicsView->transform().m11();
         if (level == 1.0) {
-            if (m_formEditorView->rootModelNode().hasAuxiliaryData("formeditorZoom"))
-                m_formEditorView->rootModelNode().setAuxiliaryData("formeditorZoom", {});
+            m_formEditorView->rootModelNode().removeAuxiliaryData(formeditorZoomProperty);
         } else {
-            m_formEditorView->rootModelNode().setAuxiliaryData("formeditorZoom", level);
+            m_formEditorView->rootModelNode().setAuxiliaryData(formeditorZoomProperty, level);
         }
     };
 
@@ -242,15 +228,15 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
         }
     };
 
-    m_zoomInAction = new QAction(zoomInIcon, tr("Zoom in"), this);
-    m_zoomInAction->setShortcut(QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Plus));
+    m_zoomInAction = new QAction(zoomInIcon, tr("Zoom In"), this);
+    m_zoomInAction->setShortcut(QKeySequence(QKeySequence::ZoomIn));
     addAction(m_zoomInAction.data());
     upperActions.append(m_zoomInAction.data());
     m_toolBox->addRightSideAction(m_zoomInAction.data());
     connect(m_zoomInAction.data(), &QAction::triggered, zoomIn);
 
-    m_zoomOutAction = new QAction(zoomOutIcon, tr("Zoom out"), this);
-    m_zoomOutAction->setShortcut(QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Minus));
+    m_zoomOutAction = new QAction(zoomOutIcon, tr("Zoom Out"), this);
+    m_zoomOutAction->setShortcut(QKeySequence(QKeySequence::ZoomOut));
     addAction(m_zoomOutAction.data());
     upperActions.append(m_zoomOutAction.data());
     m_toolBox->addRightSideAction(m_zoomOutAction.data());
@@ -262,17 +248,18 @@ FormEditorWidget::FormEditorWidget(FormEditorView *view)
     m_toolBox->addRightSideAction(m_zoomAction.data());
     connect(m_zoomAction.data(), &ZoomAction::zoomLevelChanged, setZoomLevel);
 
-    m_zoomAllAction = new QAction(zoomAllIcon, tr("Zoom screen to fit all content"), this);
-    m_zoomAllAction->setShortcut(QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_0));
+    m_zoomAllAction = new QAction(zoomAllIcon, tr("Zoom screen to fit all content."), this);
+    m_zoomAllAction->setShortcut(QKeySequence(tr("Ctrl+Alt+0")));
+
     addAction(m_zoomAllAction.data());
     upperActions.append(m_zoomAllAction.data());
     m_toolBox->addRightSideAction(m_zoomAllAction.data());
     connect(m_zoomAllAction.data(), &QAction::triggered, frameAll);
 
     m_zoomSelectionAction = new QAction(zoomSelectionIcon,
-                                        tr("Zoom screen to fit current selection"),
+                                        tr("Zoom screen to fit current selection."),
                                         this);
-    m_zoomSelectionAction->setShortcut(QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_I));
+    m_zoomSelectionAction->setShortcut(QKeySequence(tr("Ctrl+Alt+i")));
     addAction(m_zoomSelectionAction.data());
     upperActions.append(m_zoomSelectionAction.data());
     m_toolBox->addRightSideAction(m_zoomSelectionAction.data());
@@ -308,31 +295,39 @@ void FormEditorWidget::changeRootItemWidth(const QString &widthText)
 {
     bool canConvert;
     int width = widthText.toInt(&canConvert);
-    if (canConvert)
-        m_formEditorView->rootModelNode().setAuxiliaryData("width", width);
-    else
-        m_formEditorView->rootModelNode().setAuxiliaryData("width", QVariant());
+    if (canConvert) {
+        m_formEditorView->rootModelNode().setAuxiliaryData(widthProperty, width);
+    } else {
+        m_formEditorView->rootModelNode().removeAuxiliaryData(widthProperty);
+    }
 }
 
 void FormEditorWidget::changeRootItemHeight(const QString &heighText)
 {
     bool canConvert;
     int height = heighText.toInt(&canConvert);
-    if (canConvert)
-        m_formEditorView->rootModelNode().setAuxiliaryData("height", height);
-    else
-        m_formEditorView->rootModelNode().setAuxiliaryData("height", QVariant());
+    if (canConvert) {
+        m_formEditorView->rootModelNode().setAuxiliaryData(heightProperty, height);
+    } else {
+        m_formEditorView->rootModelNode().removeAuxiliaryData(heightProperty);
+    }
+}
+
+namespace {
+constexpr AuxiliaryDataKeyView formeditorColorProperty{AuxiliaryDataType::Temporary,
+                                                       "formeditorColor"};
 }
 
 void FormEditorWidget::changeBackgound(const QColor &color)
 {
     if (color.alpha() == 0) {
         m_graphicsView->activateCheckboardBackground();
-        if (m_formEditorView->rootModelNode().hasAuxiliaryData("formeditorColor"))
-            m_formEditorView->rootModelNode().setAuxiliaryData("formeditorColor", {});
+        if (m_formEditorView->rootModelNode().hasAuxiliaryData(formeditorColorProperty)) {
+            m_formEditorView->rootModelNode().setAuxiliaryData(formeditorColorProperty, {});
+        }
     } else {
         m_graphicsView->activateColoredBackground(color);
-        m_formEditorView->rootModelNode().setAuxiliaryData("formeditorColor", color);
+        m_formEditorView->rootModelNode().setAuxiliaryData(formeditorColorProperty, color);
     }
 }
 
@@ -349,10 +344,13 @@ void FormEditorWidget::initialize()
 {
     double defaultZoom = 1.0;
     if (m_formEditorView->model() && m_formEditorView->rootModelNode().isValid()) {
-        if (m_formEditorView->rootModelNode().hasAuxiliaryData("formeditorZoom"))
-            defaultZoom = m_formEditorView->rootModelNode().auxiliaryData("formeditorZoom").toDouble();
+        if (auto data = m_formEditorView->rootModelNode().auxiliaryData(formeditorZoomProperty)) {
+            defaultZoom = data->toDouble();
+        }
     }
     m_graphicsView->setZoomFactor(defaultZoom);
+    if (m_formEditorView->scene() && m_formEditorView->scene()->rootFormEditorItem())
+        m_graphicsView->centerOn(m_formEditorView->scene()->rootFormEditorItem());
     m_zoomAction->setZoomFactor(defaultZoom);
     updateActions();
 }
@@ -360,25 +358,23 @@ void FormEditorWidget::initialize()
 void FormEditorWidget::updateActions()
 {
     if (m_formEditorView->model() && m_formEditorView->rootModelNode().isValid()) {
-        if (m_formEditorView->rootModelNode().hasAuxiliaryData("width")
-            && m_formEditorView->rootModelNode().auxiliaryData("width").isValid())
-            m_rootWidthAction->setLineEditText(
-                m_formEditorView->rootModelNode().auxiliaryData("width").toString());
-        else
+        if (auto data = m_formEditorView->rootModelNode().auxiliaryData(widthProperty)) {
+            m_rootWidthAction->setLineEditText(data->toString());
+        } else {
             m_rootWidthAction->clearLineEditText();
-        if (m_formEditorView->rootModelNode().hasAuxiliaryData("height")
-            && m_formEditorView->rootModelNode().auxiliaryData("height").isValid())
-            m_rootHeightAction->setLineEditText(
-                m_formEditorView->rootModelNode().auxiliaryData("height").toString());
-        else
+        }
+
+        if (auto data = m_formEditorView->rootModelNode().auxiliaryData(heightProperty)) {
+            m_rootHeightAction->setLineEditText(data->toString());
+        } else {
             m_rootHeightAction->clearLineEditText();
+        }
 
-        if (m_formEditorView->rootModelNode().hasAuxiliaryData("formeditorColor"))
-            m_backgroundAction->setColor(
-                m_formEditorView->rootModelNode().auxiliaryData("formeditorColor").value<QColor>());
-        else
+        if (auto data = m_formEditorView->rootModelNode().auxiliaryData(formeditorColorProperty)) {
+            m_backgroundAction->setColor(data->value<QColor>());
+        } else {
             m_backgroundAction->setColor(Qt::transparent);
-
+        }
     } else {
         m_rootWidthAction->clearLineEditText();
         m_rootHeightAction->clearLineEditText();
@@ -473,18 +469,18 @@ ToolBox *FormEditorWidget::toolBox() const
 
 double FormEditorWidget::spacing() const
 {
-    return DesignerSettings::getValue(DesignerSettingsKey::ITEMSPACING).toDouble();
+    return QmlDesignerPlugin::settings().value(DesignerSettingsKey::ITEMSPACING).toDouble();
 }
 
 double FormEditorWidget::containerPadding() const
 {
-    return DesignerSettings::getValue(DesignerSettingsKey::CONTAINERPADDING).toDouble();
+    return QmlDesignerPlugin::settings().value(DesignerSettingsKey::CONTAINERPADDING).toDouble();
 }
 
 void FormEditorWidget::contextHelp(const Core::IContext::HelpCallback &callback) const
 {
     if (m_formEditorView)
-        m_formEditorView->contextHelp(callback);
+        QmlDesignerPlugin::contextHelp(callback, m_formEditorView->contextHelpId());
     else
         callback({});
 }
@@ -545,6 +541,11 @@ FormEditorGraphicsView *FormEditorWidget::graphicsView() const
     return m_graphicsView;
 }
 
+bool FormEditorWidget::errorMessageBoxIsVisible() const
+{
+    return m_documentErrorWidget && m_documentErrorWidget->isVisible();
+}
+
 DocumentWarningWidget *FormEditorWidget::errorWidget()
 {
     if (m_documentErrorWidget.isNull()) {
@@ -576,7 +577,44 @@ void FormEditorWidget::showEvent(QShowEvent *event)
         m_formEditorView->cleanupToolsAndScene();
         m_formEditorView->setupFormEditorWidget();
         m_formEditorView->resetToSelectionTool();
+        QmlItemNode rootNode = m_formEditorView->rootModelNode();
+        if (rootNode.isValid())
+            setRootItemRect(rootNode.instanceBoundingRect());
     }
 }
 
+void FormEditorWidget::dragEnterEvent(QDragEnterEvent *dragEnterEvent)
+{
+    const DesignerActionManager &actionManager = QmlDesignerPlugin::instance()
+                                                     ->viewManager().designerActionManager();
+    if (actionManager.externalDragHasSupportedAssets(dragEnterEvent->mimeData()))
+        dragEnterEvent->acceptProposedAction();
+}
+
+void FormEditorWidget::dropEvent(QDropEvent *dropEvent)
+{
+    const DesignerActionManager &actionManager = QmlDesignerPlugin::instance()
+                                                     ->viewManager().designerActionManager();
+    QHash<QString, QStringList> addedAssets = actionManager.handleExternalAssetsDrop(dropEvent->mimeData());
+
+    m_formEditorView->executeInTransaction("FormEditorWidget::dropEvent", [&] {
+        // Create Image components for added image assets
+        const QStringList addedImages = addedAssets.value(ComponentCoreConstants::addImagesDisplayString);
+        for (const QString &imgPath : addedImages) {
+            QmlItemNode::createQmlItemNodeFromImage(m_formEditorView, imgPath, {},
+                                                    m_formEditorView->scene()->rootFormEditorItem()->qmlItemNode(),
+                                                    false);
+        }
+
+        // Create Text components for added font assets
+        const QStringList addedFonts = addedAssets.value(ComponentCoreConstants::addFontsDisplayString);
+        for (const QString &fontPath : addedFonts) {
+            QString fontFamily = QFileInfo(fontPath).baseName();
+            QmlItemNode::createQmlItemNodeFromFont(m_formEditorView, fontFamily, rootItemRect().center(),
+                                                   m_formEditorView->scene()->rootFormEditorItem()->qmlItemNode(),
+                                                   false);
+        }
+    });
+}
+
 } // namespace QmlDesigner
